@@ -1,5 +1,5 @@
 """
-Copyright (c) 2012-2021 Oscar Riveros [oscar.riveros@peqnp.science].
+Copyright (c) 2012-2021 Oscar Riveros [SAT-X].
 
 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
@@ -44,7 +44,12 @@ class ALU:
         self.true = self.add_variable()
         self.false = -self.true
         self.constants = {}
-        self.add_block([-self.true])
+        if self.cnf != '':
+            self.cnf_file.write(' '.join(list(map(str, [-self.true]))) + ' 0\n')
+            self.number_of_clauses += 1
+        else:
+            raise Exception('No cnf file specified...')
+        self.signed = False
 
     @property
     def zero(self):
@@ -72,6 +77,12 @@ class ALU:
 
     def add_block(self, clause):
         clause = sorted(set(clause), key=abs)
+        if self.true == clause[0]:
+            clause.remove(self.true)
+        if self.false == clause[0]:
+            return clause
+        if not clause:
+            return clause
         if self.cnf != '':
             self.cnf_file.write(' '.join(list(map(str, clause))) + ' 0\n')
         else:
@@ -120,6 +131,26 @@ class ALU:
             value //= 2
         return block
 
+    def binary_or(self, ab, ol=None):
+        if ol is None:
+            ol = self.add_variable()
+        a, b = ab
+        self.add_block([a, b, -ol])
+        self.add_block([-a, b, ol])
+        self.add_block([a, -b, ol])
+        self.add_block([-a, -b, ol])
+        return ol
+
+    def binary_and(self, ab, ol=None):
+        if ol is None:
+            ol = self.add_variable()
+        a, b = ab
+        self.add_block([a, b, -ol])
+        self.add_block([-a, b, -ol])
+        self.add_block([a, -b, -ol])
+        self.add_block([-a, -b, ol])
+        return ol
+
     def or_gate(self, il, ol=None):
         if ol is None:
             ol = self.add_variable()
@@ -203,11 +234,11 @@ class ALU:
         return [bge((lhs, rhs), ol) for lhs, rhs, ol in zip(lhs_il, rhs_il, ol)]
 
     def bv_and_gate(self, lhs_il, rhs_il, ol=None):
-        ol = self.gate_vector(self.and_gate, lhs_il, rhs_il, ol)
+        ol = self.gate_vector(self.binary_and, lhs_il, rhs_il, ol)
         return ol
 
     def bv_or_gate(self, lhs_il, rhs_il, ol=None):
-        return self.gate_vector(self.or_gate, lhs_il, rhs_il, ol)
+        return self.gate_vector(self.binary_or, lhs_il, rhs_il, ol)
 
     def bv_xor_gate(self, lhs_il, rhs_il, ol=None):
         return self.gate_vector(self.binary_xor_gate, lhs_il, rhs_il, ol)
